@@ -10,21 +10,52 @@ from multiprocessing import Pool, cpu_count
 from tqdm.auto import tqdm
 
 from .ext_exception import ExtException
+from .json_container_decoder import JsonContainerDecoder, BigBase64
+
+
+def brace_file_read(path, file_name):
+    _path = os.path.normpath(os.path.join(path, file_name))
+    try:
+        for code_page in ['utf-8-sig', 'windows-1251']:
+            try:
+                with open(_path, 'r', encoding=code_page) as file:
+                    decoder = JsonContainerDecoder(src_dir=path, file_name=file_name)
+                    data = decoder.decode_file(file)
+                    return data
+            except UnicodeDecodeError:
+                continue
+        raise ExtException(message=f'Unknown code page in file {file_name}')
+    except (BigBase64, FileNotFoundError) as err:
+        raise err from err
+    except Exception as err:
+        raise ExtException(parent=err, message='Ошибка чтения', detail=f'{err} в файле ({_path})')
+
+
+def brace_file_write(data, path, file_name):
+    _path = os.path.normpath(os.path.join(path, file_name))
+    makedirs(path, exist_ok=True)
+    try:
+        with open(_path, 'w', encoding='utf-8') as file:
+            decoder = JsonContainerDecoder(src_dir=path, file_name=file_name)
+            raw_data = decoder.encode_root_object(data)
+            decoder.write_data(path, file_name, raw_data)
+    except Exception as err:
+        raise ExtException(message='Ошибка записи', detail=f'{err} в файле ({_path})')
 
 
 def json_read(path, file_name):
-    _path = os.path.join(path, file_name)
+    _path = os.path.normpath(os.path.join(path, file_name))
     try:
         with open(_path, 'r', encoding='utf-8') as file:
             return json.load(file)
     except FileNotFoundError as err:
-        raise err from err
+        raise err
     except Exception as err:
         raise ExtException(message='Ошибка чтения', detail=f'{err} в файле ({_path})')
 
 
 def json_write(data, path, file_name):
-    _path = os.path.join(path, file_name)
+    _path = os.path.normpath(os.path.join(path, file_name))
     makedirs(path, exist_ok=True)
     try:
         with open(_path, 'w', encoding='utf-8') as file:
@@ -36,14 +67,14 @@ def json_write(data, path, file_name):
 def txt_read(path, file_name, encoding='utf-8-sig'):
     try:
         return txt_read_detect_encoding(path, file_name, encoding=encoding)[0]
-    except FileNotFoundError as err:
+    except (FileNotFoundError, UnicodeDecodeError) as err:
         raise err from err
     except Exception as err:
-        raise ExtException(message='Ошибка чтения', detail=f'{err} в файле ({file_name})')
+        raise ExtException(parent=err, message='Ошибка чтения', detail=f'{err} в файле ({file_name})')
 
 
-def txt_read_detect_encoding(path, file_name, encoding='utf-8'):
-    _path = os.path.join(path, file_name)
+def txt_read_detect_encoding(path, file_name, encoding=None):
+    _path = os.path.normpath(os.path.join(path, file_name))
     if encoding is None:
         encoding = detect_by_bom(_path, 'utf-8')
     with open(_path, 'r', encoding=encoding) as file:
@@ -54,7 +85,7 @@ def txt_write(data, path, file_name, encoding='utf-8'):
     try:
         if data is None:
             return
-        _path = os.path.join(path, file_name)
+        _path = os.path.normpath(os.path.join(path, file_name))
         makedirs(path, exist_ok=True)
         for i in range(3):
             try:
@@ -69,37 +100,58 @@ def txt_write(data, path, file_name, encoding='utf-8'):
 
 
 def bin_write(data, path, file_name):
-    _path = os.path.join(path, file_name)
+    _path = os.path.normpath(os.path.join(path, file_name))
     makedirs(path, exist_ok=True)
     with open(_path, 'wb') as file:
         file.write(data)
 
 
 def bin_read(path, file_name):
-    _path = os.path.join(path, file_name)
+    _path = os.path.normpath(os.path.join(path, file_name))
     with open(_path, 'rb') as file:
         return file.read()
 
 
-def decode_header(obj: dict, header: list):
+def decode_header(meta_obj, header: list, *, id_in_separate_file=True):
+    obj = meta_obj.header
     try:
         obj['uuid'] = header[1][2]
         uuid.UUID(obj['uuid'])
-
     except (ValueError, IndexError):
         raise ValueError('Заголовок определен не верно')
 
+    prefix = meta_obj.options.get('prefix', '')
     obj['name'] = str_decode(header[2])
+    if prefix and obj['name'].startswith(prefix):
+        obj['name'] = obj['name'][len(prefix):]
+        header[2] = str_encode(obj['name'])
     obj['name2'] = {}
     count_locale = int(header[3][0])
     for i in range(count_locale):
         obj['name2'][str_decode(header[3][i * 2 + 1])] = str_decode(header[3][i * 2 + 2])
-    comment = str_decode(header[4]).split(';')[0]  # удаляем имя файла и номер версии которую добавляем при сборке
-    header[4] = str_encode(comment)
-    obj['comment'] = comment
-    obj['h1_0'] = header[1][0]
-    obj['h0'] = header[0]
-    obj['h5'] = header[5:]
+    # comment = str_decode(header[4]).split(';')  # удаляем имя файла и номер версии которую добавляем при сборке
+    # if len(comment) > 1:
+    #     comment[0] += ';'
+    # comment = comment[0]
+    # header[4] = str_encode(comment)
+    obj['comment'] = str_decode(header[4])
+    # obj['h1_0'] = header[1][0]
+    # obj['h0'] = header[0]
+    # obj['h5'] = header[5:]
+    if id_in_separate_file:
+        header[1][2] = 'в отдельном файле'
+        # header[2] = 'в отдельном файле'
+
+
+def encode_header(meta_obj, header: list):
+    obj = meta_obj.header
+    options = meta_obj.options
+
+    header[1][2] = obj['uuid']
+    # если собирается с параметром префикс и это объект верхнего уровня и это не заимствованный объект
+    if options and meta_obj.parent_id.find('/') == -1 and (len(header) < 6 or header[5] == '0'):
+        prefix = options.get('prefix', '')
+        header[2] = str_encode(f"{prefix}{obj['name']}")
 
 
 def clear_dir(path: str) -> None:
@@ -140,7 +192,6 @@ def close_pool(local_pool: Pool, pool: Pool = None) -> None:
 def run_in_pool(method, list_args, pool=None, title=None, need_result=False):
     _pool = get_pool(pool=pool)
     result = []
-    # msg = f'pool {method}({len(list_args)})'
     try:
         with tqdm(desc=title, total=len(list_args)) as pbar:
             for _res in _pool.imap_unordered(method, list_args, chunksize=1):
@@ -151,11 +202,68 @@ def run_in_pool(method, list_args, pool=None, title=None, need_result=False):
         raise ExtException(
             parent=err,
             action=f'run_in_pool {method.__qualname__}') from err
+    finally:
+        close_pool(_pool, pool)
+    return result
+
+
+def file_size(file):
+    """
+    Возвращает размер file-like объекта
+
+    :param file: объекта файла
+    :type file: BufferedReader
+    :return: размер в байтах
+    :rtype: int
+    """
+    pos = file.tell()
+    file.seek(0, os.SEEK_END)
+    size = file.tell()
+    file.seek(pos)
+    return size
+
+
+def run_in_pool_encode_include(method, list_args, pool=None, title=None):
+    _pool = get_pool(pool=pool)
+    file_list = []
+    include_index = {}
+    object_task = []
+    child_tasks = []
+    try:
+        with tqdm(desc=title, total=len(list_args)) as pbar:
+            for _object_task, _child_tasks in _pool.imap_unordered(method, list_args, chunksize=1):
+                if _child_tasks:
+                    child_tasks.extend(_child_tasks)
+                if isinstance(_object_task, list):
+                    object_task.append(_object_task)
+                elif isinstance(_object_task, dict):
+                    parent_id = _object_task['parent_id']
+                    obj_data = _object_task['obj_data']
+
+                    if _object_task['file_list']:
+                        file_list.extend(_object_task['file_list'])
+                    if obj_data:
+                        obj_uuid = _object_task['obj_uuid']
+                        obj_type = _object_task['obj_type']
+                        if parent_id not in include_index:
+                            include_index[parent_id] = {}
+                        if obj_type not in include_index[parent_id]:
+                            include_index[parent_id][obj_type] = []
+                        include_index[parent_id][obj_type].append((obj_uuid, _object_task['obj_name'], obj_data))
+                elif _object_task is None:
+                    pass
+                else:
+                    raise NotImplementedError()
+                pbar.update()
+    except ExtException as err:
+        raise ExtException(
+            parent=err,
+            action=f'run_in_pool {method.__qualname__}') from err
     # except Exception as err:
     #     raise ExtException(parent=err, detail=f'{method.__qualname__ {err.message}' action=f'run_in_pool {method.__qualname__}') from err
     finally:
         close_pool(_pool, pool)
-    return result
+    return file_list, include_index, object_task, child_tasks
 
 
 def list_merge(*args):
@@ -332,7 +440,7 @@ def _update_dict(base, new, _path=''):
                             dump={'value': str(base[element])}
                         )
                 elif isinstance(new[element], list):
-                    raise NotImplementedError('update list')
+                    base[element].extend(new[element])
                     # base[element] = ArrayHelper.unique_extend(base[element], new[element])
                 else:
                     base[element] = new[element]
@@ -375,13 +483,52 @@ def load_json(filename):
 
 def check_index(index_filename):
     if index_filename:
-        index = []
-        _index = load_json(index_filename)
-        sub_index = _index.pop('index.json', None)
+        try:
+            index = []
+            _index = load_json(index_filename)
+            sub_index = _index.pop('index.json', None)
+        except Exception as err:
+            raise ExtException(parent=err, message='Индексный файл не загружен', detail=index_filename)
         if sub_index:
             for elem in sub_index:
-                index.append(load_json(elem))
+                try:
+                    index.append(load_json(elem))
+                except Exception as err:
+                    raise ExtException(parent=err, message='Вложенный индексный файл не загружен', detail=elem)
         index.append(_index)
         data = update_dict(*index)
         return data
     return None
+
+
+def get_options_param(options, param_name, default=None):
+    try:
+        return options[param_name]
+    except (KeyError, TypeError):
+        return default
+
+
+def set_options_param(options, param_name, param_value):
+    if options is None:
+        options = {}
+    options[param_name] = param_value
+    return options
+
+
+def calc_offset(counters, raw_data):
+    # counters - позиции указывающие на счетчики, если не 0 то за ним идет столько записей размера size
+    #  [(3, 1), (1, 0)] (смещение относительно предыдущей записи, количество записей в единице)
+    index = 0
+    for counter_index, size in counters:
+        index += counter_index
+        if size:
+            try:
+                value = int(raw_data[index])
+            except Exception as err:
+                raise ExtException(
+                    message='bad offset',
+                    detail=f'{counter_index}={index}',
+                    dump={'counters': counters, 'value': raw_data[index]}
+                )
+            index += value * size
+    return index
