@@ -1,5 +1,4 @@
 import argparse
-import json
 import os
 import shutil
 import sys
@@ -11,18 +10,19 @@ from .container_reader import extract as container_extract, decompress_and_extra
 from .container_writer import build as container_build, compress_and_build
 from .decoder import decode, encode
 from .ext_exception import ExtException
-from .file_organizer import FileOrganizer
-from .file_organizer_ce import FileOrganizerCE
+from .organizer_file import OrganizerFile
+from .organizer_file_ce import OrganizerFileCE
 from .helper import check_index, load_json
 from .index import update_index
-from .json_container_decoder import json_decode, json_encode
 from .version import __version__
 
 
-def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, version=None, descent=None):
+def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, processes=None, options=None):
     try:
         begin0 = datetime.now()
-        print(f"v8unpack {__version__}")
+        if options is None:
+            options = dict()
+        print(f"saby v8unpack {__version__}")
 
         index = check_index(index)
         if not index and index is not None:
@@ -30,6 +30,8 @@ def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, v
 
         print(f"Начали")
 
+        descent = options.get('descent') if options else None
+
         if descent is None:
             helper.clear_dir(os.path.normpath(out_dir_name))
         clear_temp_dir = False
@@ -40,22 +42,22 @@ def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, v
 
         dir_stage0 = os.path.join(temp_dir, 'decode_stage_0')
         dir_stage1 = os.path.join(temp_dir, 'decode_stage_1')
-        dir_stage2 = os.path.join(temp_dir, 'decode_stage_2')
+        # dir_stage2 = os.path.join(temp_dir, 'decode_stage_2')
         dir_stage3 = os.path.join(temp_dir, 'decode_stage_3')
 
-        pool = helper.get_pool()
+        pool = helper.get_pool(processes=processes)
 
         container_extract(in_filename, dir_stage0, False, False)
         decompress_and_extract(dir_stage0, dir_stage1, pool=pool)
 
-        json_decode(dir_stage1, dir_stage2, pool=pool)
+        # json_decode(dir_stage1, dir_stage2, pool=pool)
 
-        decode(dir_stage2, dir_stage3, pool=pool, version=version)
+        decode(dir_stage1, dir_stage3, pool=pool, options=options)
 
         if descent is not None:
-            FileOrganizerCE.unpack(dir_stage3, out_dir_name, pool=pool, index=index, descent=int(descent))
+            OrganizerFileCE.unpack(dir_stage3, out_dir_name, pool=pool, index=index, descent=descent)
         else:
-            FileOrganizer.unpack(dir_stage3, out_dir_name, pool=pool, index=index)
+            OrganizerFile.unpack(dir_stage3, out_dir_name, pool=pool, index=index)
 
         end = datetime.now()
         print(f'{"Готово":30}: {end - begin0}')
@@ -69,11 +71,13 @@ def extract(in_filename: str, out_dir_name: str, *, temp_dir=None, index=None, v
 
 
 def build(in_dir_name: str, out_file_name: str, *, temp_dir=None, index=None,
-          version='803', descent=None, gui=None, **kwargs):
+          options=None, processes: int = None):
     try:
         begin0 = datetime.now()
+        if options is None:
+            options = {}
 
-        print(f"v8unpack {__version__}")
+        print(f"saby v8unpack {__version__}")
 
         index = check_index(index)
         if not index and index is not None:
@@ -89,20 +93,20 @@ def build(in_dir_name: str, out_file_name: str, *, temp_dir=None, index=None,
 
         dir_stage0 = os.path.join(temp_dir, 'encode_stage_0')
         dir_stage1 = os.path.join(temp_dir, 'encode_stage_1')
-        dir_stage2 = os.path.join(temp_dir, 'encode_stage_2')
+        # dir_stage2 = os.path.join(temp_dir, 'encode_stage_2')
         dir_stage3 = os.path.join(temp_dir, 'encode_stage_3')
 
-        pool = helper.get_pool(processes=1)
+        pool = helper.get_pool(processes=processes)
 
-        if descent is not None:
-            FileOrganizerCE.pack(in_dir_name, dir_stage3, pool=pool, index=index, descent=int(descent))
+        descent = options.get('descent') if options else None
+        if descent is None:
+            OrganizerFile.pack(in_dir_name, dir_stage3, pool=pool, index=index)
         else:
-            FileOrganizer.pack(in_dir_name, dir_stage3, pool=pool, index=index)
+            OrganizerFileCE.pack(in_dir_name, dir_stage3, pool=pool, index=index, descent=descent)
 
-        encode(dir_stage3, dir_stage2, version=version, pool=pool, gui=gui,
-               file_name=os.path.basename(out_file_name), **kwargs)
+        encode(dir_stage3, dir_stage1, pool=pool, file_name=os.path.basename(out_file_name), options=options)
 
-        json_encode(dir_stage2, dir_stage1, pool=pool)
+        # json_encode(dir_stage2, dir_stage1, pool=pool)
 
         compress_and_build(dir_stage1, dir_stage0, pool=pool)
         container_build(dir_stage0, out_file_name, True)
@@ -119,7 +123,7 @@ def build(in_dir_name: str, out_file_name: str, *, temp_dir=None, index=None,
         print(f'\n\n{error}')
 
 
-def build_all(product_file_name: str, product_code: str = None):
+def build_all(product_file_name: str, product_code: str = None, processes=None):
     try:
         products = load_json(os.path.abspath(product_file_name))
         if product_code:
@@ -129,12 +133,12 @@ def build_all(product_file_name: str, product_code: str = None):
             if params.get('disable'):
                 continue
             print(f'\nСобираем {product}')
+            options = params.get('options', {})
+            options['product'] = product
             build(
                 params['src'], params['bin'],
                 temp_dir=params.get('temp'), index=params.get('index'),
-                version=params.get('version'), descent=params.get('descent'),
-                gui=params.get('gui'),
-                product=product
+                options=options, processes=processes
             )
         pass
     except Exception as err:
@@ -142,7 +146,7 @@ def build_all(product_file_name: str, product_code: str = None):
         print(f'\n\n{error}')
 
 
-def extract_all(product_file_name: str, product_code: str = None):
+def extract_all(product_file_name: str, product_code: str = None, processes=None):
     try:
         products = load_json(os.path.abspath(product_file_name))
         if product_code:
@@ -155,7 +159,7 @@ def extract_all(product_file_name: str, product_code: str = None):
             extract(
                 params['bin'], params['src'],
                 temp_dir=params.get('temp'), index=params.get('index'),
-                version=params.get('version'), descent=params.get('descent')
+                options=params.get('options'), processes=processes
             )
         pass
     except Exception as err:
@@ -203,24 +207,35 @@ def main():
                             'src - путь до папки с исходниками'
                        )
     parser.add_argument('--temp', help='путь до временной папки')
-    parser.add_argument('--core', help='название общей папки добавляемой в индекс по умолчанию')
+    parser.add_argument('--prefix', help='строка добавляемая к имени метаданных 1 уровня')
+    parser.add_argument('--core', help='название общей папки добавляемой в индекс по умолчанию, только для режима -I')
     parser.add_argument('--index', help='путь до json файла с словарем копирования,'
                                         'структура файла: {путь исходника: путь общей папки}')
     parser.add_argument('--version', default='803',
-                        help="версия сборки, для сборки обработок указывается версия платформы 801/802/803, "
-                             " для сборки расширений указывается версия режима совместимости, "
+                        help="версия сборки, необязательно. "
+                             " При сборке расширений указывается если нужно задать версию режима совместимости, "
                              "например для 8.3.6 это 80306, подробности в документации на github")
-    parser.add_argument('--descent',
+    parser.add_argument('--processes', type=int,
+                        help="количество процессов которые будут использоваться для работы, "
+                             " если не указано, то количество процессов = количеству ядер компьютера. "
+                             " Нет смысла указывать значения превышающее количество ядер")
+    parser.add_argument('--descent', type=int,
                         help="включает режим наследования при сборке и разборке,"
                              "четырех значный формат 3.0.75.100 (не более 3 знаков на каждый разряд)"
                              "подробности в инструкции - раздел разработка расширений")
-    parser.add_argument('--gui',
-                        help="режим совместимости интерфейса 1С, переопределяет значение из исходников "
-                             "для расширений, если указан устанавливается в соответствующий реквизит. "
-                             "Допустимые значения: "
-                             " 0 - Версия 8.2, 1 - Версия 8.2. Разрешить Такси,"
-                             " 2- Такси. Разрешить Версия 8.2, 3 - Такси"
-                             "для расширений устанавливается в соответствующий реквизит")
+    # parser.add_argument('--gui',
+    #                     help="режим совместимости интерфейса 1С, переопределяет значение из исходников "
+    #                          "для расширений, если указан устанавливается в соответствующий реквизит. "
+    #                          "Допустимые значения: "
+    #                          " 0 - Версия 8.2, 1 - Версия 8.2. Разрешить Такси,"
+    #                          " 2- Такси. Разрешить Версия 8.2, 3 - Такси")
+    parser.add_argument("--auto_include", default=False, action="store_true",
+                        help="Если указан содержимое метаданных собирается динамически в зависимости "
+                             "от наличия файлов в каталоге, а при разборке оглавление не сохраняется, "
+                             "например вы можете подкинуть файлы метаданных в каталог из другого проекта "
+                             "минусом является, что вложенные ресурсы всегда сортируются по алфавиту. "
+                             "Если не указан, то сборщик не трогает оглавление, вы добавляете и убираете"
+                             "вложенные объекты всегда через конфигуратор")
 
     group.add_argument('-EA', nargs=1, metavar='file',
                        help='разобрать один или несколько файлов 1С, где '
@@ -238,16 +253,22 @@ def main():
         return 1
 
     args = parser.parse_args()
-    gui = args.gui if args.gui else None
+
+    options = {}
+    options_name = ['prefix', 'auto_include', 'descent', 'version']
+    for elem in options_name:
+        value = getattr(args, elem, None)
+        if value:
+            options[elem] = value
 
     if args.E is not None:
         extract(os.path.abspath(args.E[0]), os.path.abspath(args.E[1]),
-                index=args.index, temp_dir=args.temp, version=args.version, descent=args.descent)
+                index=args.index, temp_dir=args.temp, options=options)
         return
 
     if args.B is not None:
         build(os.path.abspath(args.B[0]), os.path.abspath(args.B[1]),
-              index=args.index, temp_dir=args.temp, version=args.version, descent=args.descent, gui=gui)
+              index=args.index, temp_dir=args.temp, options=options)
         return
 
     if args.BA is not None:
